#!/usr/bin/perl
#
#   gaiji.pl
#   (C) 2008 Zelgadiss
#


use Image::Magick;

use vars qw($font);

require "$ARGV[0]";


#$font = '/cygdrive/c/WINDOWS/Fonts/msgothic.ttc';
#$font = '/cygdrive/c/WINDOWS/Fonts/msmincho.ttc';
#$font = '/cygdrive/c/WINDOWS/Fonts/times.ttf';
#$font = './osaka.unicode.ttf';
#$font = './l_10646.ttf';
#$font = './bdfUMplus.ttf';
#$font = './DoulosSILR.ttf';

# 発音記号とそれ以外で文字コードが違う
my %psymbol_table = (
    '\x7c' => '{hc-p-u-ca8a}',      # ʊ [ca8a]
    '\x3a' => '{hc-p-u-3a}',        # : [cb90]
    '\x3b' => '{hc-p-u-cb8c}',      # 下付きの点 アクセント用
    '\x22' => '{hc-p-u-cb88}',      # 上付きの点 アクセント用
    '\xc3\x81' => '{hc-p-u-c381}',  # ʊə （上下に並べる）
    '\xc3\x82' => '{hc-p-u-c382}',  # ɪə （上下に並べる）
    '\xc3\x8a' => '{hc-pi-u-c999}', # əのイタリック [c999]
    '\xc3\x90' => '{hc-p-u-ca83}',  # ʃ [ca83]
    '\xc3\x98' => '{hc-p-u-ceb8}',  # θ [ceb8]
    '\xc3\xa1' => '{hc-p-u-c3a1}',  # ||
    '\xc3\xa3' => '{hc-p-u-c992}',  # ɒ [c992]
    '\xc3\xa4' => '{hc-p-u-c991}',  # ɑ [c991]
    '\xc3\xa9' => '{hc-p-u-c994}',  # ɔ [c994]
    '\xc3\xae' => '{hc-p-u-c99c}',  # ɜ  [c99c]
    '\xc3\xbb' => '{hc-p-u-c9aa}',  # ɪ [c9aa]
    '\xc3\xa6' => '{hc-p-u-c3a6}',  # æ [c3a6]
    '\xc3\xab' => '{hc-p-u-c999}',  # ə [c999]
    '\xc5\xa1' => '{hc-p-u-ca8c}',  # ʌ [ca8c]
    '\xc5\xbe' => '{hc-p-u-ca92}',  # ʒ [ca92]
    '\xca\x8c' => '{hc-p-u-ca8c}',  # ʌ
    '\xc3\xb0' => '{hc-p-u-c3b0}',  # ð
    '\xc3\x85' => '{hc-p-u-c58b}',  # ŋ
    '\xc3\x83' => '{hc-p-u-c3b0}',  # ð
    );
my %symbol_table = (
    '\xe2\x80\xa7' => '{fc-u-e280a7}', # ・
    '\xe2\x86\xaa' => '{fc-u-e286aa}', # 曲がった矢印
    '\xe2\x9e\x94' => '{fc-u-e29e94}', # 太い矢印
#    '\xc3\x80' => '{fc-u-c380}',      # A
    '\xc3\x81' => '{hc-u-c381}',       # ʊə （上下に並べる）
    '\xc3\x82' => '{hc-u-c382}',       # ɪə （上下に並べる）
#    '\xc3\x83' => '{fc-u-c383}',      # A
#    '\xc3\x84' => '{fc-u-c384}',      # A
    '\xc3\x85' => '{hc-u-c58b}',       # ŋ [c58b]
    '\xc3\x89' => '{hc-u-c389}',       # É [c389]
    '\xc3\x8a' => '{hc-u-c38a}',       # əのイタリック [c999]
    '\xc3\x90' => '{hc-u-c390}',       # ʃ [ca83]
    '\xc3\x97' => '{fc-u-c397}',       # ×
    '\xc3\x98' => '{hc-u-c398}',       # θ [ceb8]
    '\xc3\xa0' => '{hc-u-c3a0}',       # à [c3a0]
    '\xc3\xa1' => '{hc-u-c3a1}',       # a [c3a1]
    '\xc3\xa2' => '{hc-u-c3a2}',       # a [c3a2]
    '\xc3\xa3' => '{hc-u-c3a3}',       # ɒ [c992]
    '\xc3\xa4' => '{hc-u-c3a4}',       # ɑ [c991]
#    '\xc3\xa5' => '{hc-u-c991}',      # ɑ [c991]
    '\xc3\xa8' => '{hc-u-c3a8}',       # e [c3a8]
    '\xc3\xa9' => '{hc-u-c3a9}',       # é [c3a9]
    '\xc3\xaa' => '{hc-u-c3aa}',       # ê [c3aa]
    '\xc3\xad' => '{hc-u-c3ad}',       # í [c3ad]
    '\xc3\xaf' => '{hc-u-c3af}',       # i [c3af]
    '\xc3\xae' => '{hc-u-c99c}',       # ɜ [c99c]
    '\xc3\xb1' => '{hc-u-c3b1}',       # ñ [c3b1]
    '\xc3\xb3' => '{hc-u-c3b3}',       # ó [c3b3]
    '\xc3\xbb' => '{hc-u-c3bb}',       # 
    '\xc3\xa6' => '{hc-u-c3a6}',       # æ [c3a6]
    '\xc3\xab' => '{hc-u-c999}',       # ə [c999]
    '\xc5\xa1' => '{hc-u-ca8c}',       # ʌ [ca8c]
    '\xc5\xbe' => '{hc-u-ca92}',       # ʒ [ca92]
    '\xc5\x8b' => '{hc-u-c58b}',       # ŋ [c58b]
    '\xc5\x99' => '{hc-u-c599}',       # ř [c599]
    '\xc5\x9f' => '{hc-u-c59f}',       # ş [c59f]
    '\xc2\xa0' => '{hc-u-c2a0}',       # 謎の文字コード 全角スペース？
    '\xc2\xa3' => '{hc-u-c2a3}',       # £
    '\xca\x8a' => '{hc-u-ca8c}',       # ʊ
    '\xe2\x80\xa2' => '{fc-u-e280a2}', # ・
    '\xe2\x80\x94' => '{fc-u-e28094}', # －（全角マイナス）
    '\xca\x8c' => '{hc-u-ca8c}',       # ʌ
    '\xcb\x8c' => '{hc-u-cb8c}',       # 下付きの点（アクセント用？）
    '\xcb\x88' => '{hc-u-cb88}',       # 上付きの点（アクセント用？）
    '\xcb\x90' => '{hc-u-cb90}',       # ・（半角の点）
#    '\xe2\x82\x82' => '<sub>2</sub>', # 下付きの2
    '\xe2\x89\xa0' => '{fc-u-e289a0}', # ≠
    '\xe2\x86\x94' => '{fc-u-e28694}', # <->
    '\xe2\x84\xa2' => '{fc-u-e284a2}', # ™
    '\xe2\x9c\x93' => '{fc-u-e29c93}', # ✓
    '\xe3\x80\x83' => '{fc-u-e38083}', # 〃
    '\xee\x80\x95' => '{fc-u-ee8095}', # ・
    '\xcf\x80' => '{fc-u-cf80}',       # π
    '\xc9\x92' => '{fc-u-c992}',       # ɒ
    '\xc9\x99' => '{hc-u-c999}',       # eが逆さになった文字
    '\x20' => '{hc-u-20}',             # 半角スペース
    );

$x = 0;  $y = 0;

my ($image);
%fc_table = ();
%hc_table = ();

foreach $code (keys %symbol_table) {
    $tag = $symbol_table{$code};
    $skewX = 0.0;
    $base = "";
    if ($tag =~ /\{(..)-(u-(.+))\}/) {
        $case = $1;
        $base = $2;
        $code = $3;
    }
    elsif ($tag =~ /\{(..)-(p-u-(.+))\}/) {
        $case = $1;
        $base = $2;
        $code = $3;
    }
    elsif ($tag =~ /\{(..)-(pi-u-(.+))\}/) {
        $case = $1;
        $base = $2;
        $code = $3;
        $skewX = -20.0;
    }
    $text = pack('H*', $code);
    print "$text [$code]\n";
    if ($case eq 'fc') {
        $image = Image::Magick->new(size=>"16x16", pointsize=>15, font=>$font);
    }
    elsif ($case eq 'hc') {
        $image = Image::Magick->new(size=>"8x16", pointsize=>15, font=>$font);
    }
    #$image = Image::Magick->new(size=>"16x16", font=>$font);
    $image->Read('xc:white');
    $image->Annotate(text => $text,
                     x => $x, y => $y,
                     fill => 'black',
                     antialias => false,
                     style => Normal,
                     gravity => Center,
                     skewX => $skewX,
                     encoding => 'UTF-8');

    $filename = "$base.bmp";
    $image->Write("bitmap/$filename");
    $filename = "$base.xbm";
    $image->Write("gaiji/$filename");
    if ($case eq 'fc') {
        #print FCOUT "$base gaiji/$filename\n";
        $fc_table{$base} = 1;
    }
    elsif ($case eq 'hc') {
        #print HCOUT "$base gaiji/$filename\n";
        $hc_table{$base} = 1;
    }
}
foreach $code (keys %psymbol_table) {
    $tag = $psymbol_table{$code};
    $skewX = 0.0;
    $base = "";
    if ($tag =~ /\{(..)-(u-(.+))\}/) {
        $case = $1;
        $base = $2;
        $code = $3;
    }
    elsif ($tag =~ /\{(..)-(p-u-(.+))\}/) {
        $case = $1;
        $base = $2;
        $code = $3;
    }
    elsif ($tag =~ /\{(..)-(pi-u-(.+))\}/) {
        $case = $1;
        $base = $2;
        $code = $3;
        $skewX = -20.0;
    }
    $text = pack('H*', $code);
    print "$text [$code]\n";
    if ($case eq 'fc') {
        $image = Image::Magick->new(size=>"16x16", pointsize=>16, font=>$font);
    }
    elsif ($case eq 'hc') {
        $image = Image::Magick->new(size=>"8x16", pointsize=>16, font=>$font);
    }
    #$image = Image::Magick->new(size=>"16x16", font=>$font);
    $image->Read('xc:white');
    $image->Annotate(text => $text,
                     x => $x, y => $y,
                     fill => 'black',
                     antialias => false,
                     style => Normal,
                     gravity => Center,
                     skewX => $skewX,
                     encoding => 'UTF-8');

    $filename = "$base.bmp";
    $image->Write("bitmap/$filename");
    $filename = "$base.xbm";
    $image->Write("gaiji/$filename");
    if ($case eq 'fc') {
        #print FCOUT "$base gaiji/$filename\n";
        $fc_table{$base} = 1;
    }
    elsif ($case eq 'hc') {
        #print HCOUT "$base gaiji/$filename\n";
        $hc_table{$base} = 1;
    }
}
close OUT;

open(FCOUT, '>fullchar.txt') || die "Could not open file.\n";
open(HCOUT, '>halfchar.txt') || die "Could not open file.\n";

foreach $base (keys %fc_table) {
    print FCOUT "$base gaiji/$base.xbm\n";
}

foreach $base (keys %hc_table) {
    print HCOUT "$base gaiji/$base.xbm\n";
}

$image = Image::Magick->new(size=>"8x16");
$image->Read('bitmap/high_accent.bmp');
$image->Write('gaiji/u-cb88.xbm');
$image->Write('gaiji/p-u-cb88.xbm');

$image = Image::Magick->new(size=>"8x16");
$image->Read('bitmap/low_accent.bmp');
$image->Write('gaiji/u-cb8c.xbm');
$image->Write('gaiji/p-u-cb8c.xbm');

$image = Image::Magick->new(size=>"8x16");
$image->Read('bitmap/separator.bmp');
$image->Write('gaiji/p-u-c3a1.xbm');
$image = Image::Magick->new(size=>"8x16");
$image->Read('bitmap/tri_colon.bmp');
$image->Write('gaiji/p-u-3a.xbm');
$image->Write('gaiji/u-cb90.xbm');

$image = Image::Magick->new(size=>"8x16", pointsize=>12, font=>$font);
$image->Read('xc:white');
$text = pack('H*', 'ca8a');
$image->Annotate(text => $text,
                 x => 0, y => 8,
                 fill => 'black',
                 antialias => false,
                 style => Normal,
                 #gravity => Center,
                 encoding => 'UTF-8');
$text = pack('H*', 'c999');
$image->Annotate(text => $text,
                 x => 0, y => 16,
                 fill => 'black',
                 antialias => false,
                 style => Normal,
                 #gravity => Center,
                 encoding => 'UTF-8');
$image->Write('bitmap/p-u-c381.bmp');
$image->Write('gaiji/p-u-c381.xbm');

$image = Image::Magick->new(size=>"8x16", pointsize=>12, font=>$font);
$image->Read('xc:white');
$text = pack('H*', 'c9aa');
$image->Annotate(text => $text,
                 x => 0, y => 8,
                 fill => 'black',
                 antialias => false,
                 style => Normal,
                 #gravity => Center,
                 encoding => 'UTF-8');
$text = pack('H*', 'c999');
$image->Annotate(text => $text,
                 x => 0, y => 16,
                 fill => 'black',
                 antialias => false,
                 style => Normal,
                 #gravity => Center,
                 encoding => 'UTF-8');
$image->Write('bitmap/p-u-c382.bmp');
$image->Write('gaiji/p-u-c382.xbm');
