#!/usr/bin/perl
#
#   ldoce4v2-fpw.pl
#   (C) 2008 Zelgadiss
#

use strict;

use FreePWING::FPWUtils::FPWParser;
use FileHandle;
use Compress::Raw::Zlib;

require './parse.pl';

use vars qw($srcdir);

my $debug = 0;

my %dic_data;

MAIN: {
    my $time = time;
    my $page_count = 0;
    my $entry_count = 0;

    if ( $#ARGV < 0) {
        print "usage: ldoce4v2-fpw.pl conf_file\n";
        die "\n";
    }

    require "$ARGV[0]";

    my ($fpwtext, $fpwheading, $fpwword2, $fpwkeyword);
    initialize_fpwparser('text' => \$fpwtext,
                         'heading' => \$fpwheading,
                         'word2' => \$fpwword2);

    #my $srcdir = $ARGV[1];
    $srcdir =~ s/^(.+?)\/?$/$1\//;

    my $content_filename = $srcdir.'CONTENT.tda';
    my $content_index_filename = $srcdir.'CONTENT.tda.tdz';

    my $content_handle = new FileHandle;
    if (!$content_handle->open("$content_filename", 'r')) {
        die "Failed to open the file: $content_filename\n";
    }
    binmode $content_handle;

    my $content_index_handle = new FileHandle;
    if (!$content_index_handle->open("$content_index_filename", 'r')) {
        die "Failed to open the file: $content_index_filename\n";
    }
    binmode $content_index_handle;

    my $heading_position;
    my $text_position;

    for (;;) {
        my $size;
        my $contents;
        my $content;
        my $zipped_contents;
        my $tmp;
        my $headword;
        my $wordclass;
        my $info;

        if (read($content_index_handle, $tmp, 8) != 8) {
            last;
        }
        ($size) = unpack("x4V", $tmp);

        if (read($content_handle, $zipped_contents, $size) != $size) {
            die "File reading error: $content_filename\n";
        }

        my ($inflater, $status) = new Compress::Raw::Zlib::Inflate();
        if ($status != Z_OK) {
            die "Failed to initialize inflater\n";
        }
        $inflater->inflate($zipped_contents, $contents);
        if ($status != Z_OK && $status != Z_STREAM_END) {
            die "Failed to inflate\n";
        }

        foreach $content (split(/\0+/, $contents)) {
            #if ($content =~ /<td class=\"hdSel\"/) {
            {
                #unless ($content =~ /quarter<\?SK \/W\?>-<\?SK W\?>final/) {
                #    next;
                #}
                # number ̃^O\G[ɂȂĂ̂ő̃f[^Œu
                if ($content =~ /<fthwd><span id=\"p008-000251251\" class=\"HWD\"><\?SK W\?>number<\?SK \/W\?><\/span><\/fthwd>/) {
                    if (open(MODIN, '<number.xml')) {
                        $content = "";
                        while (<MODIN>) {
                            $content = $content . $_;
                        }
                    }
                    else {
                        next;
                    }
                }
                
                my ($headword, $info) = get_content($content);
                #$headword = replace_symbol($headword);
                #$headword = replace_symbol_search_word($headword);
                $info = replace_symbol($info);

                if (defined($dic_data{$headword})) {
                    $dic_data{$headword} = $dic_data{$headword} . "<br><br>" . $info;
                }
                else {
                    $dic_data{$headword} = $info;
                }
            }
        }
        my $num = keys(%dic_data);
        $| = 1;
        print "\r$num entries.";
        if ($num > 100) {
            #last;
        }
    }

    my $headword;
    foreach $headword (sort keys %dic_data) {
        my $info = $dic_data{$headword};
        my $search_word = replace_symbol_search_word($headword);
        my $headword = replace_symbol($headword);
        
        my @sentences = split(/\n/, $info);
        
        #my $num = unpack("H*", $headword);
        if ($debug) {
            print "-------------------------------------------------\n";
            print "$headword\n";
            print "$info\n";
        }

        ## {ƌoVGgɐ؂ւ
        $fpwtext->new_entry() || die $fpwtext->error_message() . "\n";
        $fpwheading->new_entry() || die $fpwheading->error_message() . "\n";

        
        ## o
        #if (!$fpwheading->add_text($headword)) {
        #    print "[$headword]\n";
        #    print_debug($headword);
        #    die $fpwheading->error_message() . "\n";
        #}
        register_text(\$fpwheading, $headword);
        
        ## {
        #if (!$fpwtext->add_keyword_start()
        #    || !$fpwtext->add_text($headword)
        #    || !$fpwtext->add_keyword_end()
        #    || !$fpwtext->add_newline()) {
        #    print "$headword\n";
        #    die $fpwtext->error_message() . "\n";
        #}
        if (!$fpwtext->add_keyword_start() ||
            !register_text(\$fpwtext, $headword) ||
            !$fpwtext->add_keyword_end() ||
            !$fpwtext->add_newline()) {
            print "$headword\n";
            die $fpwtext->error_message() . "\n";
        }
        
        # o^
        $heading_position = $fpwheading->entry_position();
        $text_position = $fpwtext->entry_position();
        if (!$fpwword2->add_entry($search_word, $heading_position,
                                  $text_position)) {
            print "[$search_word]\n";
            die $fpwword2->error_message() . "\n";
        }
        
        # {
        register_text(\$fpwtext, $info);
        $fpwtext->add_newline();
    }

    finalize_fpwparser('text' => \$fpwtext,
                       'heading' => \$fpwheading,
                       'word2' => \$fpwword2);
}

exit(0);

sub replace_tag_symbol {
    my($str) = shift @_;

    $str =~ s/&lt;/</g;
    $str =~ s/&gt;/>/g;
    $str =~ s/&amp;/&/g;

    return $str;
}

sub register_text {
    my($fpwtext, $str) = @_;
    my @token;
    my $text;

    $str =~ s/<bd><br>/<br><bd>/g;
    $str =~ s/!!\{hc-u-20\}/!!/g;
    $str =~ s/\{hc-u-20\}\{hc-u-20\}/\{hc-u-20\}/g;
    while ($str =~ /(([^<>\{\}]+)|(<.+?>)|(\{.+?\}))(.*)/) {
        push(@token, $1);
        $str = $5;
    }

    #print join(" | ", @token), "\n";
    foreach $text (@token) {
        if ($text =~ /<sub>/) {
            if (!$$fpwtext->add_subscript_start()) {
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /<\/sub>/) {
            if (!$$fpwtext->add_subscript_end()) {
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /<sup>/) {
            if (!$$fpwtext->add_superscript_start()) {
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /<\/sup>/) {
            if (!$$fpwtext->add_superscript_end()) {
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /<bd>/) {
            if (!$$fpwtext->add_font_start('bold')) {
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /<\/bd>/) {
            if (!$$fpwtext->add_font_end()) {
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /<it>/) {
            if (!$$fpwtext->add_font_start('italic')) {
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /<\/it>/) {
            if (!$$fpwtext->add_font_end()) {
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /<em>/) {
            if (!$$fpwtext->add_emphasis_start()) {
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /<\/em>/) {
            if (!$$fpwtext->add_emphasis_end()) {
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /<br>/) {
            if (!$$fpwtext->add_newline()) {
                print "[$text]\n";
                die $$fpwtext->error_massage() . "\n";
            }
        }
        elsif ($text =~ /\{fc-(.+?)\}/) {
            if (!$$fpwtext->add_full_user_character($1)) {
                print "$text [$1]\n";
                die $$fpwtext->error_message() . "\n";
            }
        }
        elsif ($text =~ /\{hc-(.+?)\}/) {
            if (!$$fpwtext->add_half_user_character($1)) {
                print "$text [$1]\n";
                die $$fpwtext->error_message() . "\n";
            }
        }
        else {
            if (!$$fpwtext->add_text($text)) {
                print "[$text]\n";
                print_debug($text);
                die $$fpwtext->error_message() . "\n";
            }
        }
    }

    return 1;
}
